/// <reference types="node" />
import { EventEmitter } from "events";
import { IOptions, IPipe, IPlan, IState } from "./interfaces";
/**
 * class of NodeSpider
 * @class NodeSpider
 */
export default class NodeSpider extends EventEmitter {
    _STATE: IState;
    /**
     * create an instance of NodeSpider
     * @param option
     */
    constructor(option?: IOptions);
    /**
     * Check whether the url has been added
     * @param {string} url
     * @returns {boolean}
     */
    has(url: string): boolean;
    /**
     * 过滤掉一个数组中的重复链接，以及所有已被添加的链接，返回一个新数组
     * @param urls {array}
     * @returns {array}
     */
    filter(urls: string[]): string[];
    /**
     * add new plan
     * @param  {IPlan}  plan plan object
     * @return {void}
     */
    plan(plan: IPlan): void;
    /**
     * connect new pipe
     * @param  {IPipe}  target pipe object
     * @return {void}
     */
    pipe(newPipe: IPipe): void;
    /**
     * add new tasks, return tasks' uuids
     * @param planName target plan name
     * @param url url(s)
     * @param info attached information
     */
    add(planName: string, url: string | string[], info?: {
        [index: string]: any;
    }): string[];
    /**
     * filter new tasks and add, return tasks' uuids
     * @param planName target plan name
     * @param url url(s)
     * @param info attached information
     */
    addU(planName: string, url: string | string[], info?: {
        [index: string]: any;
    }): string[];
    /**
     * Save data through a pipe
     * @param  {string} pipeName pipe name
     * @param  {any}    data     data you need to save
     * @return {void}
     */
    save(pipeName: string, data: {
        [index: string]: any;
    }): void;
    pause(): void;
    active(): void;
    end(): void;
}
